# Copyright (C) 2001  Miklos Reiter (see the file README for details on
# how to contact me)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#

# Pass this program two arguments:
# -the first one should be the name of a file where the $1's store a list of classes to look for, separated by newlines
# -the second one the (fully qualified) name of a java class
# -the third one the name of the corresponding java source file
#
# The program outputs a list of java classes also contained in the first file and needed by the given
# class, separated by newlines, to stdout.





# global arrays used:
# look_for: array indexed by fully qualified classnames to be searched, stores short names
# as stored by look_for_short, concatenated, separated by " "
# look_for_short: array indexed by short class names to be found, storing the corresponding long classnames


#--------------------------------------INITIALIZATION--------------------------------------------------


# initializing variables and processing arguments
BEGIN { doInit() }



#---------------------------------------FUNCTIONS------------------------------------------------------



# initializes global variables:
function doInit()
{
  ml_comment = 0;
  package_open = 0;
  import_open = 0;
  class_name = ARGV[2];
  ARGV[2] = ""; # this is not a file to process
  look_for_file = ARGV[1]; # file storing all the classes to search the target class for dependencies for
}

# deals with multi-line comments in the current $0
function doml()
{
  if (ml_comment)
  {
    if (ml_comment_end = index($0, "*\/"))
    {
      # remove start of string
      $0 = substr($0, ml_comment_end + 2);
      ml_comment = 0;
      doml(); # now parse for further comments
    }
    else $0 = ""
  }
  else
  while (ml_comment_start = index($0, "\/*"))
  {
    if (ml_comment_end = index($0, "*\/"))
    {
      # remove string between ml_comment_start and ml_comment_end
      $0 = substr($0, 1, ml_comment_start - 1) substr($0, ml_comment_end + 2)
    }
    else
    {
      # remove end of string after ml_comment_start and signal that we're inside a multi-line comment
      $0 = substr($0, 1, ml_comment_start - 1);
      ml_comment = 1;
      return
    }
  }
}


# get package name in $0, starting at start
function get_package(start)
{
    p_str = substr($0, start);
    if (match(p_str, /\y(\w+\.)*\w+\y/))
      {
        p_str = substr(p_str, RSTART, RLENGTH);
        found_package(p_str ".");
        package_open = 0;
      }
    else package_open = 1
}

# get imported class name or package.* in $0, starting at start
function get_import(start)
{
    p_str = substr($0, start);

    if (match(p_str, /\y(\w+\.)+\*/))
      {
        p_str = substr(p_str, RSTART, RLENGTH - 1);
        found_package(p_str); # . at end already included
        package_open = 0;
      }
    else if (match(p_str, /\y(\w+\.)*\w+\y/)) # single imported class
      # this does not mean the class is used, it just defines its short name
      {
        long_name = substr(p_str, RSTART, RLENGTH);
        if (long_name in look_for)
        {
          short_name = long_name;
          gsub(/[^\.]+\./, "", short_name);
          look_for[long_name] = look_for[long_name] " " short_name;
          look_for_short[short_name] = long_name;
        }
        import_open = 0;
      }
    else import_open = 1
}

# add package to list of packages that are imported
function found_package(package,                       short_name)
{
  # add resulting short names to look_for_short
  for (class in look_for)
  {
    if (index(class, package) == 1)
    {
      short_name = substr(class, length(package) + 1);
      look_for_short[short_name] = class;
      look_for[class] = look_for[class] short_name " ";
#      print "New short name: " short_name
    }
  }

  # for debugging
#  print "Package found: " package
}

# add class to array of found classes, class is fully qualified class name
function found_class(class,               short_names)
{
  # deal with long names
  if (class in look_for)
  {
    split(look_for[class], short_names, " ");
    delete look_for[class];
    for (short_name in short_names) delete look_for_short[short_name];
    # output class name
    print class
  }

  # deal with short names, by resolving and calling us recursively
  if (class in look_for_short)
  {
    found_class(look_for_short[class]);
    return
  }
}



#--------------------------------------FURTHER RULES-------------------------------------------


# first of all, process look_for_file and store each line in look_for; go on
# to next line: this prevents other rules from being processed for the first file
(FILENAME == look_for_file) { if ($1 != class_name) look_for[$1] = " "; next }


# strip string literals enclosed in ""'s
/"/ { gsub(/"[^"]*"/, "") }


# strip single-line comments, i.e. rest of line after //
/\/\// { sub(/\/\/.*/, "") }

# strip mult-line comments, i.e. everything from /* to */, maybe across several lines
#{ if (!ml_comment) gsub(/\/\*([^\*]*(\*+[^\/\*])?)*\*+\//, "'&'") } # works, but horrendously slow!
{ doml() } # low-level comment-removal, longer, but not any less readable;-), and fast!

# parse package statement for implicitly imported (current) package
{ 
  if (package_open)
    get_package(1)
  else
    if (match($1, /\ypackage\y/))
      get_package(RSTART + RLENGTH)
}

# parse import statements for imported classes and packages
{
  if (import_open)
    get_import(1)
  else
    if (match($1, /\yimport\y/))
      get_import(RSTART + RLENGTH)
}


# finally, the most important line,
# the one that searches the current line for occurrences of classes in look_for and look_for_short
{
  for (c in look_for) if ($0 ~ ("\\y" c "\\y")) found_class(c);
  for (c in look_for_short) if ($0 ~ ("\\y" c "\\y")) found_class(c);
}

