/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import javax.swing.border.*;

import java.lang.IllegalStateException;

import noope.core.Vector3D;
import noope.core.Physics;

import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.input.ArrayBlockReader;

/**
 * Title:        NOOPE Gravity and Sphere Collision demonstration
 * Description:  This is a demonstration of NOOPE (the Newtonian Object-oriented Physics Engine). It simulates
 * the motion of spheres under gravity and elastic collision.
 *
 * This is the main dialog containing controls for input and a button to
 * run the simulation. When the user presses this button, the second dialog
 * RunDialog is displayed.
 *
 * @author Miklos Reiter
 * @version May 2001
 */

public class Applet1 extends JApplet {

    private int numberOfOutputProjectionLaws; // needed during setting up of ArrayBlockReaders
    private String[] projectionLawNames = new String[3]; // idem, names of the OutputProjection laws.

  boolean isStandalone = false;
  Box runBox;
  JButton runButton = new JButton();
  JPanel globalPanel = new JPanel();
  Box globalBox;
  JPanel gravityPanel = new JPanel();
  Box constGBox;
  JLabel labelG = new JLabel();
  JTextField textG = new JTextField();
  TitledBorder titledBorder1;
  JPanel projectionPanel = new JPanel();
  Box projectionVertBox;
  JPanel projection1Panel = new JPanel();
  Box projection1Box;
  JLabel projection1NameLabel = new JLabel();
  JTextField textProj1Name = new JTextField();
  JLabel lblproj1Normal = new JLabel();
  JTextField textNormal1 = new JTextField();
  JPanel entitiesPanel = new JPanel();
  JLabel entitiesTitleLabel = new JLabel();
  JPanel entityDetailsPanel = new JPanel();
  Box hBoxEntity1;
  JLabel lblNameEntity1 = new JLabel();
  JTextField textNameEntity1 = new JTextField();
  JLabel lblMassEntity1 = new JLabel();
  JTextField textMassEntity1 = new JTextField();
  JLabel lblRadiusEntity1 = new JLabel();
  JTextField textRadiusEntity1 = new JTextField();
  JComboBox comboColorEntity1 = new JComboBox();
  JLabel lblColorEntitiy1 = new JLabel();
  JButton buttonChooseColorEntity1 = new JButton();
  BorderLayout borderLayout1 = new BorderLayout();
  JLabel lblxEntity1 = new JLabel();
  JTextField textxEntity1 = new JTextField();
  JLabel lblvEntity1 = new JLabel();
  JTextField textvEntity1 = new JTextField();
  JComboBox comboColorEntity2 = new JComboBox();
  JTextField textvEntity2 = new JTextField();
  JLabel lblvEntity2 = new JLabel();
  JLabel lblMassEntity2 = new JLabel();
  JTextField textNameEntity2 = new JTextField();
  JTextField textRadiusEntity2 = new JTextField();
  JTextField textMassEntity2 = new JTextField();
  JLabel lblColorEntitiy2 = new JLabel();
  JButton buttonChooseColorEntity2 = new JButton();
  JTextField textxEntity2 = new JTextField();
  JLabel lblxEntity2 = new JLabel();
  JLabel lblRadiusEntity2 = new JLabel();
  JLabel lblNameEntity2 = new JLabel();
  Box hBoxEntity2;
  JComboBox comboColorEntity3 = new JComboBox();
  JTextField textvEntity3 = new JTextField();
  JLabel lblvEntity3 = new JLabel();
  JLabel lblMassEntity3 = new JLabel();
  JTextField textNameEntity3 = new JTextField();
  JTextField textRadiusEntity3 = new JTextField();
  JTextField textMassEntity3 = new JTextField();
  JLabel lblColorEntitiy3 = new JLabel();
  JButton buttonChooseColorEntity3 = new JButton();
  JTextField textxEntity3 = new JTextField();
  JLabel lblxEntity3 = new JLabel();
  JLabel lblRadiusEntity3 = new JLabel();
  JLabel lblNameEntity3 = new JLabel();
  Box hBoxEntity3;
  JComboBox comboColorEntity4 = new JComboBox();
  JTextField textvEntity4 = new JTextField();
  JLabel lblvEntity4 = new JLabel();
  JLabel lblMassEntity4 = new JLabel();
  JTextField textNameEntity4 = new JTextField();
  JTextField textRadiusEntity4 = new JTextField();
  JTextField textMassEntity4 = new JTextField();
  JLabel lblColorEntitiy4 = new JLabel();
  JButton buttonChooseColorEntity4 = new JButton();
  JTextField textxEntity4 = new JTextField();
  JLabel lblxEntity4 = new JLabel();
  JLabel lblRadiusEntity4 = new JLabel();
  JLabel lblNameEntity4 = new JLabel();
  Box hBoxEntity4;
  JComboBox comboColorEntity5 = new JComboBox();
  JTextField textvEntity5 = new JTextField();
  JLabel lblvEntity5 = new JLabel();
  JLabel lblMassEntity5 = new JLabel();
  JTextField textNameEntity5 = new JTextField();
  JTextField textRadiusEntity5 = new JTextField();
  JTextField textMassEntity5 = new JTextField();
  JLabel lblColorEntitiy5 = new JLabel();
  JButton buttonChooseColorEntity5 = new JButton();
  JTextField textxEntity5 = new JTextField();
  JLabel lblxEntity5 = new JLabel();
  JLabel lblRadiusEntity5 = new JLabel();
  JLabel lblNameEntity5 = new JLabel();
  Box hBoxEntity5;
  JLabel globalTitleLabel = new JLabel();
  BorderLayout borderLayout2 = new BorderLayout();
  JLabel lblx1 = new JLabel();
  JTextField textxmin1 = new JTextField();
  JLabel lblxDash1 = new JLabel();
  JTextField textxmax1 = new JTextField();
  JLabel lbly1 = new JLabel();
  JTextField textymin1 = new JTextField();
  JLabel lblyDash1 = new JLabel();
  JTextField textymax1 = new JTextField();
  JTextField textxmax2 = new JTextField();
  JLabel lblyDash2 = new JLabel();
  JTextField textymin2 = new JTextField();
  JLabel lbly2 = new JLabel();
  JTextField textxmin2 = new JTextField();
  JLabel projection1NameLabel1 = new JLabel();
  Box projection1Box1;
  JPanel projection1Panel1 = new JPanel();
  JTextField textNormal2 = new JTextField();
  JLabel lblx2 = new JLabel();
  JLabel lblproj1Normal1 = new JLabel();
  JLabel lblxDash2 = new JLabel();
  JTextField textymax2 = new JTextField();
  JTextField textProj1Name1 = new JTextField();
  JTextField textxmax3 = new JTextField();
  JLabel lblyDash3 = new JLabel();
  JTextField textymin3 = new JTextField();
  JLabel lbly3 = new JLabel();
  JTextField textxmin3 = new JTextField();
  JLabel projection1NameLabel2 = new JLabel();
  Box projection1Box2;
  JPanel projection1Panel2 = new JPanel();
  JTextField textNormal3 = new JTextField();
  JLabel lblx3 = new JLabel();
  JLabel lblproj1Normal2 = new JLabel();
  JLabel lblxDash3 = new JLabel();
  JTextField textymax3 = new JTextField();
  JTextField textProj1Name2 = new JTextField();
  JLabel projectionLabel = new JLabel();
  BorderLayout borderLayout3 = new BorderLayout();
  /**Get a parameter value*/
  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (getParameter(key) != null ? getParameter(key) : def);
  }

  /**Construct the applet*/
  public Applet1() {
  }
  /**Initialize the applet*/
  public void init() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**Component initialization*/
  private void jbInit() throws Exception {
    runBox = Box.createHorizontalBox();
    globalBox = Box.createVerticalBox();
    constGBox = Box.createHorizontalBox();
    titledBorder1 = new TitledBorder("");
    projectionVertBox = Box.createVerticalBox();
    projection1Box = Box.createHorizontalBox();
    hBoxEntity1 = Box.createHorizontalBox();
    hBoxEntity2 = Box.createHorizontalBox();
    hBoxEntity3 = Box.createHorizontalBox();
    hBoxEntity4 = Box.createHorizontalBox();
    hBoxEntity5 = Box.createHorizontalBox();
    projection1Box1 = Box.createHorizontalBox();
    projection1Box2 = Box.createHorizontalBox();
    //    this.setSize(new Dimension(599, 400));
    runButton.setText("Go!");
    runButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runButton_actionPerformed(e);
      }
    });
    globalPanel.setBorder(BorderFactory.createEtchedBorder());
    globalPanel.setLayout(borderLayout2);
    labelG.setLabelFor(textG);
    labelG.setText("G = ");
    textG.setText("6.6732e-11");
    gravityPanel.setBorder(BorderFactory.createEtchedBorder());
    projectionPanel.setBorder(BorderFactory.createEtchedBorder());
    projectionPanel.setLayout(borderLayout3);
    projection1NameLabel.setLabelFor(textProj1Name);
    projection1NameLabel.setText("Name: ");
    textProj1Name.setPreferredSize(new Dimension(65, 19));
    textProj1Name.setText("x-y plane");
    lblproj1Normal.setLabelFor(textNormal1);
    lblproj1Normal.setText("  n = ");
    textNormal1.setPreferredSize(new Dimension(60, 19));
    textNormal1.setText("(0,0,1)");
    entitiesPanel.setBorder(BorderFactory.createEtchedBorder());
    entitiesPanel.setLayout(borderLayout1);
    entitiesTitleLabel.setFont(new java.awt.Font("SansSerif", 1, 15));
    entitiesTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    entitiesTitleLabel.setText("Entities");
    lblNameEntity1.setLabelFor(textNameEntity1);
    lblNameEntity1.setText("Name: ");
    textNameEntity1.setPreferredSize(new Dimension(45, 19));
    textNameEntity1.setText("Entity1");
    lblMassEntity1.setLabelFor(textMassEntity1);
    lblMassEntity1.setText("  m = ");
    textMassEntity1.setPreferredSize(new Dimension(45, 19));
    textMassEntity1.setText("1.0e10");
    lblRadiusEntity1.setLabelFor(textRadiusEntity1);
    lblRadiusEntity1.setText("  r = ");
    textRadiusEntity1.setPreferredSize(new Dimension(45, 19));
    textRadiusEntity1.setText("0.1");
    lblColorEntitiy1.setLabelFor(comboColorEntity1);
    lblColorEntitiy1.setText("  Color = ");
    comboColorEntity1.setMinimumSize(new Dimension(80, 24));
    comboColorEntity1.setPreferredSize(new Dimension(80, 24));
    comboColorEntity1.setEditable(true);
    comboColorEntity1.setMaximumRowCount(16);
    buttonChooseColorEntity1.setText("...");
    buttonChooseColorEntity1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonChooseColorEntity1_actionPerformed(e);
      }
    });
    lblxEntity1.setLabelFor(textxEntity1);
    lblxEntity1.setText("  x = ");
    textxEntity1.setPreferredSize(new Dimension(45, 19));
    textxEntity1.setText("(1,0,0)");
    lblvEntity1.setLabelFor(textvEntity1);
    lblvEntity1.setText("  v = ");
    textvEntity1.setPreferredSize(new Dimension(45, 19));
    textvEntity1.setText("0");
    comboColorEntity2.setMaximumRowCount(16);
    comboColorEntity2.setEditable(true);
    comboColorEntity2.setPreferredSize(new Dimension(80, 24));
    comboColorEntity2.setMinimumSize(new Dimension(80, 24));
    textvEntity2.setPreferredSize(new Dimension(45, 19));
    textvEntity2.setText("(0,0.5,0.25)");
    lblvEntity2.setText("  v = ");
    lblvEntity2.setLabelFor(textvEntity2);
    lblMassEntity2.setText("  m = ");
    lblMassEntity2.setLabelFor(textMassEntity2);
    textNameEntity2.setText("Entity2");
    textNameEntity2.setPreferredSize(new Dimension(45, 19));
    textRadiusEntity2.setText("0.2");
    textRadiusEntity2.setPreferredSize(new Dimension(45, 19));
    textMassEntity2.setText("2.0e10");
    textMassEntity2.setPreferredSize(new Dimension(45, 19));
    lblColorEntitiy2.setText("  Color = ");
    lblColorEntitiy2.setLabelFor(comboColorEntity2);
    buttonChooseColorEntity2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonChooseColorEntity2_actionPerformed(e);
      }
    });
    buttonChooseColorEntity2.setText("...");
    textxEntity2.setPreferredSize(new Dimension(45, 19));
    textxEntity2.setText("(0,0,0)");
    lblxEntity2.setText("  x = ");
    lblxEntity2.setLabelFor(textxEntity2);
    lblRadiusEntity2.setText("  r = ");
    lblRadiusEntity2.setLabelFor(textRadiusEntity2);
    lblNameEntity2.setText("Name: ");
    lblNameEntity2.setLabelFor(textNameEntity2);
    comboColorEntity3.setMaximumRowCount(16);
    comboColorEntity3.setEditable(true);
    comboColorEntity3.setPreferredSize(new Dimension(80, 24));
    comboColorEntity3.setMinimumSize(new Dimension(80, 24));
    textvEntity3.setPreferredSize(new Dimension(45, 19));
    textvEntity3.setText("(0,0,0)");
    lblvEntity3.setText("  v = ");
    lblvEntity3.setLabelFor(textvEntity3);
    lblMassEntity3.setText("  m = ");
    lblMassEntity3.setLabelFor(textMassEntity3);
    textNameEntity3.setText("Entity3");
    textNameEntity3.setPreferredSize(new Dimension(45, 19));
    textRadiusEntity3.setText("0.2");
    textRadiusEntity3.setPreferredSize(new Dimension(45, 19));
    textMassEntity3.setText("1e10");
    textMassEntity3.setPreferredSize(new Dimension(45, 19));
    lblColorEntitiy3.setText("  Color = ");
    lblColorEntitiy3.setLabelFor(comboColorEntity3);
    buttonChooseColorEntity3.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonChooseColorEntity3_actionPerformed(e);
      }
    });
    buttonChooseColorEntity3.setText("...");
    textxEntity3.setPreferredSize(new Dimension(45, 19));
    textxEntity3.setText("(-1,0,0)");
    lblxEntity3.setText("  x = ");
    lblxEntity3.setLabelFor(textxEntity3);
    lblRadiusEntity3.setText("  r = ");
    lblRadiusEntity3.setLabelFor(textRadiusEntity3);
    lblNameEntity3.setText("Name: ");
    lblNameEntity3.setLabelFor(textNameEntity3);
    comboColorEntity4.setMaximumRowCount(16);
    comboColorEntity4.setEditable(true);
    comboColorEntity4.setPreferredSize(new Dimension(80, 24));
    comboColorEntity4.setMinimumSize(new Dimension(80, 24));
    textvEntity4.setPreferredSize(new Dimension(45, 19));
    lblvEntity4.setText("  v = ");
    lblvEntity4.setLabelFor(textvEntity4);
    lblMassEntity4.setText("  m = ");
    lblMassEntity4.setLabelFor(textMassEntity4);
    textNameEntity4.setPreferredSize(new Dimension(45, 19));
    textRadiusEntity4.setPreferredSize(new Dimension(45, 19));
    textMassEntity4.setPreferredSize(new Dimension(45, 19));
    lblColorEntitiy4.setText("  Color = ");
    lblColorEntitiy4.setLabelFor(comboColorEntity4);
    buttonChooseColorEntity4.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonChooseColorEntity4_actionPerformed(e);
      }
    });
    buttonChooseColorEntity4.setText("...");
    textxEntity4.setPreferredSize(new Dimension(45, 19));
    lblxEntity4.setText("  x = ");
    lblxEntity4.setLabelFor(textxEntity4);
    lblRadiusEntity4.setText("  r = ");
    lblRadiusEntity4.setLabelFor(textRadiusEntity4);
    lblNameEntity4.setText("Name: ");
    lblNameEntity4.setLabelFor(textNameEntity4);
    comboColorEntity5.setMaximumRowCount(16);
    comboColorEntity5.setEditable(true);
    comboColorEntity5.setPreferredSize(new Dimension(80, 24));
    comboColorEntity5.setMinimumSize(new Dimension(80, 24));
    textvEntity5.setPreferredSize(new Dimension(45, 19));
    lblvEntity5.setText("  v = ");
    lblvEntity5.setLabelFor(textvEntity5);
    lblMassEntity5.setText("  m = ");
    lblMassEntity5.setLabelFor(textMassEntity5);
    textNameEntity5.setPreferredSize(new Dimension(45, 19));
    textRadiusEntity5.setPreferredSize(new Dimension(45, 19));
    textMassEntity5.setPreferredSize(new Dimension(45, 19));
    lblColorEntitiy5.setText("  Color = ");
    lblColorEntitiy5.setLabelFor(comboColorEntity5);
    buttonChooseColorEntity5.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonChooseColorEntity5_actionPerformed(e);
      }
    });
    buttonChooseColorEntity5.setText("...");
    textxEntity5.setPreferredSize(new Dimension(45, 19));
    lblxEntity5.setText("  x = ");
    lblxEntity5.setLabelFor(textxEntity5);
    lblRadiusEntity5.setText("  r = ");
    lblRadiusEntity5.setLabelFor(textRadiusEntity5);
    lblNameEntity5.setText("Name: ");
    lblNameEntity5.setLabelFor(textNameEntity5);
    globalTitleLabel.setFont(new java.awt.Font("SansSerif", 1, 15));
    globalTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    globalTitleLabel.setText("Global settings");
    lblx1.setLabelFor(textxmin1);
    lblx1.setText("  x: ");
    textxmin1.setPreferredSize(new Dimension(60, 19));
    textxmin1.setText("-4.0");
    lblxDash1.setLabelFor(textxmax1);
    lblxDash1.setText(" - ");
    textxmax1.setPreferredSize(new Dimension(60, 19));
    textxmax1.setText("4.0");
    lbly1.setLabelFor(textymin1);
    lbly1.setText("  y: ");
    textymin1.setPreferredSize(new Dimension(60, 19));
    textymin1.setText("-4.0");
    lblyDash1.setLabelFor(textymax1);
    lblyDash1.setText(" - ");
    textymax1.setPreferredSize(new Dimension(60, 19));
    textymax1.setText("4.0");
    textxmax2.setPreferredSize(new Dimension(60, 19));
    textxmax2.setText("4.0");
    lblyDash2.setText(" - ");
    lblyDash2.setLabelFor(textymax2);
    textymin2.setPreferredSize(new Dimension(60, 19));
    textymin2.setText("-4.0");
    lbly2.setText("  y: ");
    lbly2.setLabelFor(textymin2);
    textxmin2.setPreferredSize(new Dimension(60, 19));
    textxmin2.setText("-4.0");
    projection1NameLabel1.setText("Name: ");
    projection1NameLabel1.setLabelFor(textProj1Name1);
    textNormal2.setPreferredSize(new Dimension(60, 19));
    textNormal2.setText("(0,-1,0)");
    lblx2.setLabelFor(textxmin2);
    lblx2.setText("  x: ");
    lblproj1Normal1.setLabelFor(textNormal2);
    lblproj1Normal1.setText("  n = ");
    lblxDash2.setLabelFor(textxmax2);
    lblxDash2.setText(" - ");
    textymax2.setPreferredSize(new Dimension(60, 19));
    textymax2.setText("4.0");
    textProj1Name1.setPreferredSize(new Dimension(65, 19));
    textProj1Name1.setText("x-z plane");
    textxmax3.setPreferredSize(new Dimension(60, 19));
    lblyDash3.setText(" - ");
    lblyDash3.setLabelFor(textymax3);
    textymin3.setPreferredSize(new Dimension(60, 19));
    lbly3.setText("  y: ");
    lbly3.setLabelFor(textymin3);
    textxmin3.setPreferredSize(new Dimension(60, 19));
    projection1NameLabel2.setText("Name: ");
    projection1NameLabel2.setLabelFor(textProj1Name2);
    textNormal3.setPreferredSize(new Dimension(60, 19));
    lblx3.setLabelFor(textxmin3);
    lblx3.setText("  x: ");
    lblproj1Normal2.setLabelFor(textNormal3);
    lblproj1Normal2.setText("  n = ");
    lblxDash3.setLabelFor(textxmax3);
    lblxDash3.setText(" - ");
    textymax3.setPreferredSize(new Dimension(60, 19));
    textProj1Name2.setPreferredSize(new Dimension(65, 19));
    projectionLabel.setFont(new java.awt.Font("SansSerif", 1, 15));
    projectionLabel.setDisplayedMnemonic('0');
    projectionLabel.setHorizontalAlignment(SwingConstants.CENTER);
    projectionLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    projectionLabel.setText("Projections");
    this.getContentPane().add(runBox, BorderLayout.SOUTH);
    runBox.add(runButton, null);
    this.getContentPane().add(globalPanel, BorderLayout.NORTH);
    globalPanel.add(globalTitleLabel, BorderLayout.NORTH);
    globalPanel.add(globalBox, BorderLayout.CENTER);
    globalBox.add(gravityPanel, null);
    gravityPanel.add(constGBox, null);
    constGBox.add(labelG, null);
    constGBox.add(textG, null);
    globalBox.add(projectionPanel, null);
    projectionPanel.add(projectionLabel, BorderLayout.NORTH);
    projectionPanel.add(projectionVertBox, BorderLayout.CENTER);
    projectionVertBox.add(projection1Panel, null);
    projection1Panel.add(projection1Box, null);
    projection1Box.add(projection1NameLabel, null);
    projection1Box.add(textProj1Name, null);
    projection1Box.add(lblproj1Normal, null);
    projection1Box.add(textNormal1, null);
    projection1Box.add(lblx1, null);
    projection1Box.add(textxmin1, null);
    projection1Box.add(lblxDash1, null);
    projection1Box.add(textxmax1, null);
    projection1Box.add(lbly1, null);
    projection1Box.add(textymin1, null);
    projection1Box.add(lblyDash1, null);
    projection1Box.add(textymax1, null);
    projectionVertBox.add(projection1Panel1, null);
    projection1Panel1.add(projection1Box1, null);
    projection1Box1.add(projection1NameLabel1, null);
    projection1Box1.add(textProj1Name1, null);
    projection1Box1.add(lblproj1Normal1, null);
    projection1Box1.add(textNormal2, null);
    projection1Box1.add(lblx2, null);
    projection1Box1.add(textxmin2, null);
    projection1Box1.add(lblxDash2, null);
    projection1Box1.add(textxmax2, null);
    projection1Box1.add(lbly2, null);
    projection1Box1.add(textymin2, null);
    projection1Box1.add(lblyDash2, null);
    projection1Box1.add(textymax2, null);
    projectionVertBox.add(projection1Panel2, null);
    projection1Panel2.add(projection1Box2, null);
    projection1Box2.add(projection1NameLabel2, null);
    projection1Box2.add(textProj1Name2, null);
    projection1Box2.add(lblproj1Normal2, null);
    projection1Box2.add(textNormal3, null);
    projection1Box2.add(lblx3, null);
    projection1Box2.add(textxmin3, null);
    projection1Box2.add(lblxDash3, null);
    projection1Box2.add(textxmax3, null);
    projection1Box2.add(lbly3, null);
    projection1Box2.add(textymin3, null);
    projection1Box2.add(lblyDash3, null);
    projection1Box2.add(textymax3, null);
    this.getContentPane().add(entitiesPanel, BorderLayout.CENTER);
    entitiesPanel.add(entitiesTitleLabel, BorderLayout.NORTH);
    entitiesPanel.add(entityDetailsPanel, BorderLayout.CENTER);
    entityDetailsPanel.add(hBoxEntity1, null);
    hBoxEntity1.add(lblNameEntity1, null);
    hBoxEntity1.add(textNameEntity1, null);
    hBoxEntity1.add(lblMassEntity1, null);
    hBoxEntity1.add(textMassEntity1, null);
    hBoxEntity1.add(lblRadiusEntity1, null);
    hBoxEntity1.add(textRadiusEntity1, null);
    hBoxEntity1.add(lblxEntity1, null);
    hBoxEntity1.add(textxEntity1, null);
    hBoxEntity1.add(lblvEntity1, null);
    hBoxEntity1.add(textvEntity1, null);
    hBoxEntity1.add(lblColorEntitiy1, null);
    hBoxEntity1.add(comboColorEntity1, null);
    hBoxEntity1.add(buttonChooseColorEntity1, null);
    entityDetailsPanel.add(hBoxEntity2, null);
    hBoxEntity2.add(lblNameEntity2, null);
    hBoxEntity2.add(textNameEntity2, null);
    hBoxEntity2.add(lblMassEntity2, null);
    hBoxEntity2.add(textMassEntity2, null);
    hBoxEntity2.add(lblRadiusEntity2, null);
    hBoxEntity2.add(textRadiusEntity2, null);
    hBoxEntity2.add(lblxEntity2, null);
    hBoxEntity2.add(textxEntity2, null);
    hBoxEntity2.add(lblvEntity2, null);
    hBoxEntity2.add(textvEntity2, null);
    hBoxEntity2.add(lblColorEntitiy2, null);
    hBoxEntity2.add(comboColorEntity2, null);
    hBoxEntity2.add(buttonChooseColorEntity2, null);
    entityDetailsPanel.add(hBoxEntity3, null);
    hBoxEntity3.add(lblNameEntity3, null);
    hBoxEntity3.add(textNameEntity3, null);
    hBoxEntity3.add(lblMassEntity3, null);
    hBoxEntity3.add(textMassEntity3, null);
    hBoxEntity3.add(lblRadiusEntity3, null);
    hBoxEntity3.add(textRadiusEntity3, null);
    hBoxEntity3.add(lblxEntity3, null);
    hBoxEntity3.add(textxEntity3, null);
    hBoxEntity3.add(lblvEntity3, null);
    hBoxEntity3.add(textvEntity3, null);
    hBoxEntity3.add(lblColorEntitiy3, null);
    hBoxEntity3.add(comboColorEntity3, null);
    hBoxEntity3.add(buttonChooseColorEntity3, null);
    entityDetailsPanel.add(hBoxEntity4, null);
    hBoxEntity4.add(lblNameEntity4, null);
    hBoxEntity4.add(textNameEntity4, null);
    hBoxEntity4.add(lblMassEntity4, null);
    hBoxEntity4.add(textMassEntity4, null);
    hBoxEntity4.add(lblRadiusEntity4, null);
    hBoxEntity4.add(textRadiusEntity4, null);
    hBoxEntity4.add(lblxEntity4, null);
    hBoxEntity4.add(textxEntity4, null);
    hBoxEntity4.add(lblvEntity4, null);
    hBoxEntity4.add(textvEntity4, null);
    hBoxEntity4.add(lblColorEntitiy4, null);
    hBoxEntity4.add(comboColorEntity4, null);
    hBoxEntity4.add(buttonChooseColorEntity4, null);
    entityDetailsPanel.add(hBoxEntity5, null);
    hBoxEntity5.add(lblNameEntity5, null);
    hBoxEntity5.add(textNameEntity5, null);
    hBoxEntity5.add(lblMassEntity5, null);
    hBoxEntity5.add(textMassEntity5, null);
    hBoxEntity5.add(lblRadiusEntity5, null);
    hBoxEntity5.add(textRadiusEntity5, null);
    hBoxEntity5.add(lblxEntity5, null);
    hBoxEntity5.add(textxEntity5, null);
    hBoxEntity5.add(lblvEntity5, null);
    hBoxEntity5.add(textvEntity5, null);
    hBoxEntity5.add(lblColorEntitiy5, null);
    hBoxEntity5.add(comboColorEntity5, null);
    hBoxEntity5.add(buttonChooseColorEntity5, null);

    // initialize color combo with color names
    initColorCombo(comboColorEntity1);
    comboColorEntity1.setSelectedIndex(10);
    initColorCombo(comboColorEntity2);
    comboColorEntity2.setSelectedIndex(1);
    initColorCombo(comboColorEntity3);
    comboColorEntity3.setSelectedIndex(5);
    initColorCombo(comboColorEntity4);
    initColorCombo(comboColorEntity5);

    // set InputVerifier's
    // Vector3DInputVerifier
    Vector3DInputVerifier v = new Vector3DInputVerifier();

    textNormal1.setInputVerifier(v);
    textNormal2.setInputVerifier(v);
    textNormal3.setInputVerifier(v);
    textxEntity1.setInputVerifier(v);
    textxEntity2.setInputVerifier(v);
    textxEntity3.setInputVerifier(v);
    textxEntity4.setInputVerifier(v);
    textxEntity5.setInputVerifier(v);
    textvEntity1.setInputVerifier(v);
    textvEntity2.setInputVerifier(v);
    textvEntity3.setInputVerifier(v);
    textvEntity4.setInputVerifier(v);
    textvEntity5.setInputVerifier(v);

    // DoubleInputVerifier
    DoubleInputVerifier d = new DoubleInputVerifier();

    textG.setInputVerifier(d);

    textxmin1.setInputVerifier(d);
    textxmin2.setInputVerifier(d);
    textxmin3.setInputVerifier(d);

    textymin1.setInputVerifier(d);
    textymin2.setInputVerifier(d);
    textymin3.setInputVerifier(d);

    textxmax1.setInputVerifier(d);
    textxmax2.setInputVerifier(d);
    textxmax3.setInputVerifier(d);

    textymax1.setInputVerifier(d);
    textymax2.setInputVerifier(d);
    textymax3.setInputVerifier(d);

    textMassEntity1.setInputVerifier(d);
    textMassEntity2.setInputVerifier(d);
    textMassEntity3.setInputVerifier(d);
    textMassEntity4.setInputVerifier(d);
    textMassEntity5.setInputVerifier(d);

    textRadiusEntity1.setInputVerifier(d);
    textRadiusEntity2.setInputVerifier(d);
    textRadiusEntity3.setInputVerifier(d);
    textRadiusEntity4.setInputVerifier(d);
    textRadiusEntity5.setInputVerifier(d);

    // ColorInputVerifier
    ColorInputVerifier c = new ColorInputVerifier();
    ((JComponent)comboColorEntity1.getEditor().getEditorComponent()).setInputVerifier(c);
    ((JComponent)comboColorEntity2.getEditor().getEditorComponent()).setInputVerifier(c);
    ((JComponent)comboColorEntity3.getEditor().getEditorComponent()).setInputVerifier(c);
    ((JComponent)comboColorEntity4.getEditor().getEditorComponent()).setInputVerifier(c);
    ((JComponent)comboColorEntity5.getEditor().getEditorComponent()).setInputVerifier(c);

  }

  // add color names to combobox
  private void initColorCombo(JComboBox c)
  {
    c.addItem("black");
    c.addItem("blue");
    c.addItem("cyan");
    c.addItem("darkGray");
    c.addItem("gray");
    c.addItem("green");
    c.addItem("lightGray");
    c.addItem("magenta");

    c.addItem("orange");
    c.addItem("pink");
    c.addItem("red");
    c.addItem("white");
    c.addItem("yellow");
  }

  /**Start the applet*/
  public void start() {
  }
  /**Stop the applet*/
  public void stop() {
  }
  /**Destroy the applet*/
  public void destroy() {
  }
  /**Get Applet information*/
  public String getAppletInfo() {
    return "Applet Information";
  }
  /**Get parameter info*/
  public String[][] getParameterInfo() {
    return null;
  }
  /**Main method*/
  public static void main(String[] args) {
    Applet1 applet = new Applet1();
    applet.isStandalone = true;
    JFrame frame = new JFrame();
    //EXIT_ON_CLOSE == 3
    frame.setDefaultCloseOperation(3);
    frame.setTitle("Gravity and SphereCollision Demo");
    frame.getContentPane().add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    frame.setSize(640,400);
    //    frame.pack();
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }

  //static initializer for setting look & feel
  static {
    try {
      //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      //UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
    }
    catch(Exception e) {
    }
  }

  private static Color string2Color(Component parent, String s)
  {
    if (s.equals("black")) return Color.black; else if
           (s.equals("blue")) return Color.blue; else if
           (s.equals("cyan")) return Color.cyan; else if
           (s.equals("darkGray")) return Color.darkGray; else if
           (s.equals("gray")) return Color.gray; else if
           (s.equals("green")) return Color.green; else if
           (s.equals("lightGray")) return Color.lightGray; else if
           (s.equals("magenta")) return Color.magenta; else if
           (s.equals("orange")) return Color.orange; else if
           (s.equals("pink")) return Color.pink; else if
           (s.equals("red")) return Color.red; else if
           (s.equals("white")) return Color.white; else if
           (s.equals("yellow")) return Color.yellow;
    else
      try
      {
        Vector3D v = new Vector3D(s);
        return new Color((float)v.getX(), (float)v.getY(), (float)v.getZ());
      }
      catch (java.text.ParseException e)
      {
        JOptionPane.showMessageDialog(parent, "Invalid Color entered!", "Error",
				      JOptionPane.ERROR_MESSAGE);
      }
      return null;
  }

  /** gives the user the possibility to choose a color; the default
   *  color is the entry in comboColorEntity and the color chosen
   *  is entered there
   */
  private void colorButtonPressed(JComboBox comboColorEntity)
  {
    Color oldC = string2Color(this,
      comboColorEntity.getEditor().getItem().toString());
    if (oldC == null) return;

    Color c = JColorChooser.showDialog(this, "Choose Color", oldC);
    if (c != null) // has the user entered something?
    {
      String sColor = ("(" + ((float)c.getRed()/255.0F) + ","
        + ((float)c.getGreen()/255.0F) + "," +
        ((float)c.getBlue()/255.0F) + ")");
      comboColorEntity.getEditor().setItem(sColor);
    }
  }

  void buttonChooseColorEntity1_actionPerformed(ActionEvent e) {
    if (buttonChooseColorEntity1.hasFocus())
      colorButtonPressed(comboColorEntity1);
  }
  void buttonChooseColorEntity2_actionPerformed(ActionEvent e) {
    if (buttonChooseColorEntity2.hasFocus())
      colorButtonPressed(comboColorEntity2);
  }
  void buttonChooseColorEntity3_actionPerformed(ActionEvent e) {
    if (buttonChooseColorEntity3.hasFocus())
      colorButtonPressed(comboColorEntity3);
  }
  void buttonChooseColorEntity4_actionPerformed(ActionEvent e) {
    if (buttonChooseColorEntity4.hasFocus())
      colorButtonPressed(comboColorEntity4);
  }
  void buttonChooseColorEntity5_actionPerformed(ActionEvent e) {
    if (buttonChooseColorEntity5.hasFocus())
      colorButtonPressed(comboColorEntity5);
  }

    /**	Run! Displays a modal dialog. */
    void runButton_actionPerformed(ActionEvent e)
    {
	BlockReader physBr = null;
	Physics phys = null;
	if (runButton.hasFocus())
	    {
		try
		    {
			physBr = getPhysBr();
			phys = new Physics(physBr);
		    }
		catch (IllegalStateException f) // not all data entered
		    {
			// just return, a message is displayed at the appropriate place
			return;
		    }
		catch (BRLoadingException f)
		    {
			JOptionPane.showMessageDialog(this, "Error occurred while constructing"
						      + " Physics. Message: " + f, "Error",
						      JOptionPane.ERROR_MESSAGE);
			f.printStackTrace();

			if (phys != null) phys.dispose(); // tidy up phys
		       
			return;
		    }
		RunDialog runD = new RunDialog();
		runD.setModal(true);
		runD.setTitle("Simulation");
		runD.setPhysics(phys);
		runD.startSimulation();
		runD.show();
		// now free any resources associated with the dialog
		//runD.dispose();
	    }
	// otherwise, some input-verification hasn't been successful.
    }


    /* -----------------BlockReader construction functions: -------------------*/


    /** returns a BlockReader object representing a Physics object. */
    private BlockReader getPhysBr() throws IllegalStateException
    {
	String header = "physics", parameter = null;
	String[][] entries = null;
	BlockReader[] subblocks = {getLawsBr(getLawBrs()), getEntitiesBr(getEntityBrs())};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }
    
    /** returns a BlockReader object representing a Gravity, a SphereCollision and various
	OutputProjection laws. */
    private BlockReader[] getLawBrs() throws IllegalStateException
    {
	String samplingFreq = "0.01";

	// get G
	String sG = textG.getText();
	if (sG.equals("")) throw new IllegalStateException();
		
	int outLawN = ((textProj1Name.getText().equals("") ? 0 : 1)
		       + (textProj1Name1.getText().equals("") ? 0 : 1)
		       + (textProj1Name2.getText().equals("") ? 0 : 1));

	numberOfOutputProjectionLaws = outLawN; // set class data member for getEntityBrs()
	
	BlockReader[] result = new BlockReader[outLawN + 2];

	result[0] = newGravBr(sG);
	result[1] = newSphereCollBr();
	
	int i = 2;
	
	if (!(textProj1Name.getText().equals("")))
	{
	    if (textNormal1.getText().equals("") || textxmin1.getText().equals("") ||
		textxmax1.getText().equals("") || textymin1.getText().equals("") ||
		textymax1.getText().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Projection 1, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}
	    result[i] = newOutputProjectionBr
		(textProj1Name.getText(),
		 samplingFreq,
		 textNormal1.getText(),
		 textxmin1.getText(),
		 textxmax1.getText(),
		 textymin1.getText(),
		 textymax1.getText(),
		 "300", "300", "0", "0");
	    
	    projectionLawNames[i-2] = textProj1Name.getText();
	    
	    i++;
	}

	if (!(textProj1Name1.getText().equals("")))
	{
	    if (textNormal2.getText().equals("") || textxmin2.getText().equals("") ||
		textxmax2.getText().equals("") || textymin2.getText().equals("") ||
		textymax2.getText().equals("")) 
	    {
		JOptionPane.showMessageDialog(this, "If you enter the name of a Projection 2, "
					      + "please enter all other details, too.",
					      "Error", JOptionPane.ERROR_MESSAGE);
		throw new IllegalStateException();
	    }
	    result[i] = newOutputProjectionBr
		(textProj1Name1.getText(),
		 samplingFreq,
		 textNormal2.getText(),
		 textxmin2.getText(),
		 textxmax2.getText(),
		 textymin2.getText(),
		 textymax2.getText(),
		 "300", "300", "300", "0");

	    projectionLawNames[i-2] = textProj1Name1.getText();

	    i++;
	}

	if (!(textProj1Name2.getText().equals("")))
	{
	    if (textNormal3.getText().equals("") || textxmin3.getText().equals("") ||
		textxmax3.getText().equals("") || textymin3.getText().equals("") ||
		textymax3.getText().equals("")) 
	    {
		JOptionPane.showMessageDialog(this, "If you enter the name of a Projection 3, "
					      + "please enter all other details, too.",
					      "Error", JOptionPane.ERROR_MESSAGE);
		throw new IllegalStateException();
	    }
	    result[i] = newOutputProjectionBr
		(textProj1Name2.getText(),
		 samplingFreq,
		 textNormal3.getText(),
		 textxmin3.getText(),
		 textxmax3.getText(),
		 textymin3.getText(),
		 textymax3.getText(),
		 "300", "300", "0", "300");

	    projectionLawNames[i-2] = textProj1Name2.getText();

	    i++;
	}

	return result;

    }

    /** returns a BlockReader object representing the various entities. */
    private BlockReader[] getEntityBrs() throws IllegalStateException
    {
	Color col;
	String sColor;
	int entityCount = ((textNameEntity1.getText().equals("") ? 0 : 1)
			   + (textNameEntity2.getText().equals("") ? 0 : 1)
			   + (textNameEntity3.getText().equals("") ? 0 : 1)
			   + (textNameEntity4.getText().equals("") ? 0 : 1)
			   + (textNameEntity5.getText().equals("") ? 0 : 1));
	
	BlockReader[] result = new BlockReader[entityCount];
	
	int i = 0;
	if (!textNameEntity1.getText().equals(""))
	{
	    if (textMassEntity1.getText().equals("") || textRadiusEntity1.getText().equals("")
		|| textxEntity1.getText().equals("") || textvEntity1.getText().equals("")
		|| comboColorEntity1.getEditor().getItem().toString().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Entity 1, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}

	    col = string2Color(this, comboColorEntity1.getEditor().getItem().toString());
	    if (col == null) throw new IllegalStateException();
	    sColor = ("(" + ((float)col.getRed()/255.0F) + "," + ((float)col.getGreen()/255.0F) +
		      "," + ((float)col.getBlue()/255.0F) + ")");

	    result[i] = newEntityBr(textNameEntity1.getText(), textxEntity1.getText(),
				    textvEntity1.getText(), textMassEntity1.getText(),
				    textRadiusEntity1.getText(), sColor);
	    i++;
	}

	if (!textNameEntity2.getText().equals(""))
	{
	    if (textMassEntity2.getText().equals("") || textRadiusEntity2.getText().equals("")
		|| textxEntity2.getText().equals("") || textvEntity2.getText().equals("")
		|| comboColorEntity2.getEditor().getItem().toString().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Entity 2, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}
	    
	    col = string2Color(this, comboColorEntity2.getEditor().getItem().toString());
	    if (col == null) throw new IllegalStateException();
	    sColor = ("(" + ((float)col.getRed()/255.0F) + "," + ((float)col.getGreen()/255.0F) +
		      "," + ((float)col.getBlue()/255.0F) + ")");

	    result[i] = newEntityBr(textNameEntity2.getText(), textxEntity2.getText(),
				    textvEntity2.getText(), textMassEntity2.getText(),
				    textRadiusEntity2.getText(),
				    sColor);
	    i++;
	}

	if (!textNameEntity3.getText().equals(""))
	{
	    if (textMassEntity3.getText().equals("") || textRadiusEntity3.getText().equals("")
		|| textxEntity3.getText().equals("") || textvEntity3.getText().equals("")
		|| comboColorEntity3.getEditor().getItem().toString().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Entity 3, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}

	    col = string2Color(this, comboColorEntity3.getEditor().getItem().toString());
	    if (col == null) throw new IllegalStateException();
	    sColor = ("(" + ((float)col.getRed()/255.0F) + "," + ((float)col.getGreen()/255.0F) +
		      "," + ((float)col.getBlue()/255.0F) + ")");


	    result[i] = newEntityBr(textNameEntity3.getText(), textxEntity3.getText(),
				    textvEntity3.getText(), textMassEntity3.getText(),
				    textRadiusEntity3.getText(),
				    sColor);
	    i++;
	}

	if (!textNameEntity4.getText().equals(""))
	{
	    if (textMassEntity4.getText().equals("") || textRadiusEntity4.getText().equals("")
		|| textxEntity4.getText().equals("") || textvEntity4.getText().equals("")
		|| comboColorEntity4.getEditor().getItem().toString().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Entity 4, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}

	    col = string2Color(this, comboColorEntity4.getEditor().getItem().toString());
	    if (col == null) throw new IllegalStateException();
	    sColor = ("(" + ((float)col.getRed()/255.0F) + "," + ((float)col.getGreen()/255.0F) +
		      "," + ((float)col.getBlue()/255.0F) + ")");

	    result[i] = newEntityBr(textNameEntity4.getText(), textxEntity4.getText(),
				    textvEntity4.getText(), textMassEntity4.getText(),
				    textRadiusEntity4.getText(),
				    sColor);
	    i++;
	}

	if (!textNameEntity5.getText().equals(""))
	{
	    if (textMassEntity5.getText().equals("") || textRadiusEntity5.getText().equals("")
		|| textxEntity5.getText().equals("") || textvEntity5.getText().equals("")
		|| comboColorEntity5.getEditor().getItem().toString().equals(""))
		{
		    JOptionPane.showMessageDialog(this, "If you enter the name of a Entity 5, "
						  + "please enter all other details, too.",
						  "Error", JOptionPane.ERROR_MESSAGE);
		    throw new IllegalStateException();
		}

	    col = string2Color(this, comboColorEntity5.getEditor().getItem().toString());
	    if (col == null) throw new IllegalStateException();
	    sColor = ("(" + ((float)col.getRed()/255.0F) + "," + ((float)col.getGreen()/255.0F) +
		      "," + ((float)col.getBlue()/255.0F) + ")");

	    result[i] = newEntityBr(textNameEntity5.getText(), textxEntity5.getText(),
				    textvEntity5.getText(), textMassEntity5.getText(),
				    textRadiusEntity5.getText(),
				    sColor);
	    i++;
	}

	return result;
	
    }

    /** creates an ArrayBlockReader defining an Entity */
    private BlockReader newEntityBr(String name, String position, String velocity,
					   String inertialMass, String radius, String colorRGB)
	throws IllegalStateException
    {
	String header = "entity", parameter = "";
	String[][] entries = 
	{{"name", name},
	 {"position", position},
	 {"velocity", velocity},
	 {"inertialMass", inertialMass}
	};
	BlockReader[] subblocks = new BlockReader[1+numberOfOutputProjectionLaws];
	subblocks[0] = newSphereCollPropBr(radius);
	
	for (int i = 1; i <= numberOfOutputProjectionLaws; i++)
	    {
		subblocks[i] = 
		    new ArrayBlockReader(projectionLawNames[i-1], "OutputProjectionCirclePropertyRecord",
					 new String[][]{{"colorRGB", colorRGB},
							{"radius", radius}
					 }, null);
	    }
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }
    
    
    // creates the entities {...} block containing all the entities as subblocks
    private BlockReader getEntitiesBr(BlockReader[] subblocks)
    {
	String header = "entities", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    // creates the laws {...} block containing all the laws as subblocks
    private BlockReader getLawsBr(BlockReader[] subblocks)
    {
	String header = "laws", parameter = null;
	String[][] entries = null;

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }    

    private BlockReader newGravBr(String sG) // creates an ArrayBlockReader defining a Gravity law
    {
	String header = "law", parameter = "Gravity"; 
	String[][] entries = 
	{{"name", "Gravity"},
	 {"G", sG}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** Create a BlockReader object to construct a SphereCollision law from */
    private BlockReader newSphereCollBr()
    {
	String header = "law", parameter = "SphereCollision"; 
	String[][] entries = 
	{{"name", "SphereCollision"}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** creates an ArrayBlockReader defining a OutputProjection law */
    private BlockReader newOutputProjectionBr(String name, String period, String normal, 
						     String xmin,
						     String xmax, String ymin, String ymax,
						     String width, String height, String xpos,
						     String ypos)
    {
	String header = "law", parameter = "noope.output.OutputProjection";
	String[][] entries =
	{{"name", name},
	 {"period", period},
	 {"normal", normal},
	 {"xmin", xmin},
	 {"xmax", xmax},
	 {"ymin", ymin},
	 {"ymax", ymax},
	 {"width", width},
	 {"height", height},
	 {"xpos", xpos},
	 {"ypos", ypos}
	};
	BlockReader[] subblocks = {};
	
	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

    /** creates an ArrayBlockReader defining a SphereCollisionPropertyRecord */
    private BlockReader newSphereCollPropBr(String radius) 
    {
	String header = "SphereCollision", parameter = null; 
	String[][] entries = {
	    {"radius", radius}
	};
	BlockReader[] subblocks = {};

	return new ArrayBlockReader(header, parameter, entries, subblocks);
    }

}
