/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import javax.swing.*;


/**
 * This is an InputVerifier object for swing JTextField objects that
 * verifies whether the value entered can be parsed as a double.
 * @author Miklos Reiter
 * @version May 2001
 */

public class DoubleInputVerifier extends InputVerifier {
  /**
   * Whether the DoubleInputVerifier accepts the empty string as a valid
   * input.
   */
  private boolean acceptEmpty;

  /**
   * Constructs a new DoubleInputVerifier object.
   *
   * This verifier will accept the empty string as a valid entry.
   */
  public DoubleInputVerifier()
  {
    acceptEmpty = true;
  }

  /**
   * Constructs a new DoubleInputVerifier object.
   *
   * @arg acceptEmpty Whether the empty string should be accepted as
   * valid.
   */
  public DoubleInputVerifier(boolean emptyEntryAccepted) {
    acceptEmpty = emptyEntryAccepted;
  }

  /**
   * Is the empty string accepted as a valid entry?
   */
  public boolean getAcceptEmpty()
  {
    return acceptEmpty;
  }

  /**
   * Sets whether the empty string should be accepted as valid.
   */
  public void setAcceptEmpty(boolean emptyEntryAccepted)
  {
    acceptEmpty = emptyEntryAccepted;
  }

  /**
   * Displays a message box complaining to the user and informing them
   * briefly of the format of a double, if the input is invalid, in
   * addition to calling the inherited method.
   */
  public boolean shouldYieldFocus(JComponent input)
  {
    boolean result = super.shouldYieldFocus(input);
    if (!result)
      JOptionPane.showMessageDialog(input, "Please enter a valid " +
        "double. (A double is a decimal number.)",
         "Error", JOptionPane.ERROR_MESSAGE);
    return result;
  }


  /**
   *  Casts the argument to a JTextField and tries to parse its text
   *  as a double. Returns true if it succeeds, false otherwise.
   */
  public boolean verify(JComponent input) {
    String s = ((JTextField)input).getText();
    if (getAcceptEmpty() && s.equals("")) return true;
    try
    {
      double d = java.lang.Double.parseDouble(s);
    }
    catch (java.lang.NumberFormatException e)
    {
      return false;
    }
    return true;
  }
}
