/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package gravsphcolldemo;

import java.awt.*;
import java.awt.event.*;

import noope.output.MessageBox;

import java.lang.IllegalArgumentException;

/**
 * Title:        AWT Gravity and Sphere Collision Demo Applet
 * Description:  This applet demonstrates the use of NOOPE to simulate the Gravity and SphereCollision laws. It uses AWT, so it can be run in old browsers.
 *
 * For most purposes, you can use the static methods provided that create
 * modal dialogs.
 *
 * @author Miklos Reiter
 * @version 1.0
 */

public class EditProjectionDialog extends Dialog {

    /** Stores the owner; getOwner() apparently didn't exist in Java 1.1. */
    protected Frame owner;

    /**
     * Whether the dialog has been cancelled.
     */
    protected boolean cancelled;
    /**
     * Stores the data the user has entered.
     */
    protected ProjectionBrData result;
  Panel panelMain = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  Panel panelSettings = new Panel();
  Panel panelButtons = new Panel();
  Button buttonCancel = new Button();
  Button buttonOK = new Button();
  Label lblName = new Label();
  TextField txtName = new TextField();
  Label lblNormal = new Label();
  TextField txtn = new TextField();
  Label lbx = new Label();
  TextField txtxmin = new TextField();
  Label lblxTo = new Label();
  TextField txtxmax = new TextField();
  Label lbly = new Label();
  TextField txtymin = new TextField();
  Label lblyTo = new Label();
  TextField txtymax = new TextField();

  /**
   * Makes a new instance of this dialog.
   */
  protected EditProjectionDialog(Frame owner, String title) {
    super(owner, title, true); // modal!
    this.owner = owner;
    setModal(true);
    // initialize components
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Displays a new instance of this dialog, allowing the user to enter
   * data about a projection.
   */
  public static ProjectionBrData newProjectionDialog(Frame owner)
  {
    // make a new dialog
    EditProjectionDialog d = new EditProjectionDialog(owner, "New Projection");
    // execute it
    d.setVisible(true);
    d.dispose();
    if (d.cancelled)
	return null;
    else return d.result;
  }


    /**
     * Displays a new instance of this dialog, allowing the user to edit the ProjectionBrData
     * record initialValue.
     */
    public static ProjectionBrData editProjectionDialog(Frame owner, ProjectionBrData data)
    {
	if (data == null) return newProjectionDialog(owner);
	EditProjectionDialog d = new EditProjectionDialog(owner, "Edit Projection");
	d.txtName.setText(data.getName());
	d.txtn.setText(data.getNormal());
	d.txtxmin.setText(data.getXmin());
	d.txtxmax.setText(data.getXmax());
	d.txtymin.setText(data.getYmin());
	d.txtymax.setText(data.getYmax());
	d.setVisible(true);
	d.dispose();
	if (d.cancelled)
	    return null;
	else return d.result;
    }


  private void jbInit() throws Exception {
    panelMain.setLayout(borderLayout1);
    buttonCancel.setLabel("Cancel");
    buttonCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonCancel_actionPerformed(e);
      }
    });
    buttonOK.setLabel("OK");
    buttonOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        buttonOK_actionPerformed(e);
      }
    });
    lblName.setText("Name: ");
    txtName.setColumns(5);
    lblNormal.setText(" n = ");
    txtn.setColumns(5);
    lbx.setText(" x: ");
    txtxmin.setColumns(5);
    lblxTo.setText(" - ");
    txtxmax.setColumns(5);
    lbly.setText(" y: ");
    txtymin.setColumns(5);
    lblyTo.setText(" - ");
    txtymax.setColumns(5);
    panelMain.add(panelSettings, BorderLayout.CENTER);
    panelSettings.add(lblName, null);
    panelSettings.add(txtName, null);
    panelSettings.add(lblNormal, null);
    panelSettings.add(txtn, null);
    panelSettings.add(lbx, null);
    panelSettings.add(txtxmin, null);
    panelSettings.add(lblxTo, null);
    panelSettings.add(txtxmax, null);
    panelSettings.add(lbly, null);
    panelSettings.add(txtymin, null);
    panelSettings.add(lblyTo, null);
    panelSettings.add(txtymax, null);
    panelMain.add(panelButtons, BorderLayout.SOUTH);
    panelButtons.add(buttonCancel, null);
    panelButtons.add(buttonOK, null);

    // added by Miklos Reiter
    this.add(panelMain);
    this.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        buttonCancel_actionPerformed(null);
      }
    });

    // added by Miklos Reiter
    this.pack();
  }

    /**
     * This method is called when the user presses the OK button. Creates a new EntityBrData object
     * and stores it in result. If the user has not entered all the data or the data is not
     * properly formatted, a dialog box is displayed informing the user, and the focus is set
     * back to the TextField causing the trouble. If all the data is valid, cancelled is set to
     * false and the dialog is hidden.
     */
    protected void buttonOK_actionPerformed(ActionEvent e) {
	cancelled = false;
	result = new ProjectionBrData();
	
	result.setName(txtName.getText());
	
	try
	    {
		result.setNormal(txtn.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", "Please enter a valid Vector3D for n.");
		txtn.requestFocus();
		return;
	    }

	try
	    {
		result.setXmin(txtxmin.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Double for the lower x-value.");
		txtxmin.requestFocus();
		return;
	    }

	try
	    {
		result.setXmax(txtxmax.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Double for the upper x-value.");
		txtxmax.requestFocus();
		return;
	    }

	try
	    {
		result.setYmin(txtymin.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Double for the lower y-value.");
		txtymin.requestFocus();
		return;
	    }

	try
	    {
		result.setYmax(txtymax.getText());
	    }
	catch (IllegalArgumentException f)
	    {
		MessageBox.displayMessage((Frame)owner, "Error", 
					  "Please enter a valid Double for the upper y-value.");
		txtymax.requestFocus();
		return;
	    }

	this.setVisible(false);
    }

    /** Called when the user presses the Cancel button or closes the dialog. Sets cancelled to
     *	true and hides the dialog.
     */
    protected void buttonCancel_actionPerformed(ActionEvent e)
    {
	cancelled = true;
	this.setVisible(false);
    }
}
