/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.core.Physics;
import noope.laws.Law;

/**
 * Objects of this class provide to Laws a means of creating new instances of Law classes with
 * a given name. These Law classes can be dynamically loaded or hard-coded into the LawLoader
 * (e.g. for an Applet that isn't allowed by the SecurityManager to access the constructor of a
 * dynamically loaded class).
 *
 * @author Miklos Reiter
 * @version 28-Apr-2001
 *
 */
abstract public class LawLoader
{
    /**
     * This method should create a new Law object. The arguments, except for className and brL 
     * are passed
     * on to the Law constructor, so the class has to have a constructor of this form.
     *
     * @arg className The name of the class to load.
     * @arg br This is the BlockReader to use to construct the law.
     * @arg brL The BlockReaderLocation used to construct a BRLoadingException if the creation
     * of the Law fails.
     * @arg lawnumber This is the number of the law to use.
     * @arg phys This is a reference to the Physics object that is being constructed and to
     * which this law will belong.
     */
    abstract public Law createLaw(String className, BlockReaderLocation brL, BlockReader br, 
				  int lawnumber, Physics phys)
	throws BRLoadingException;
}
