/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.input;

import noope.core.Vector3D;
import java.lang.Double;
import java.awt.Color;

/**
 *
 * This class contains routines for validating various noope-related data like Vector3D, Double,
 * Color, etc.
 *
 * @author Miklos Reiter
 * @version June 2001
 */
public class Valid
{
  /**
   *  Returns true if s can be parsed as a double, false otherwise.
   */
  public static boolean validDouble(String s) {
    try
    {
      Double d = Double.valueOf(s);
    }
    catch (java.lang.NumberFormatException e)
    {
      return false;
    }
    return true;
  }

  /**
   * Returns true if the argument represents a valid Vector3D. Uses
   * the constructor of the Vector3D class taking a String.
   */
  public static boolean validVector3D(String s)
  {
    try
    {
      Vector3D v = new Vector3D(s);
    }
    catch (java.text.ParseException e)
    {
      return false;
    }
    return true; // no ParseException occurred
  }

    /** Returns true if the String s represents a valid Color object, false otherwise. */
  public static boolean validColor(String s)
  {
      if (s.equals("black") || (s.equals("blue")) || (s.equals("cyan")) 
	  || (s.equals("darkGray")) || (s.equals("gray")) || (s.equals("green"))
	  || (s.equals("lightGray")) ||  (s.equals("magenta")) || (s.equals("orange"))
	  || (s.equals("pink")) || (s.equals("red")) || (s.equals("white")) ||
	  (s.equals("yellow")))
	  return true; // one of predefined options
      
      try
	  {
	      Vector3D v = new Vector3D(s);
	      return (0.0 <= v.getX() && v.getX() <= 1.0 && 0.0 <= v.getY() && v.getY() <= 1.0
		      && 0.0 <= v.getZ() && v.getZ() <= 1.0);
	      // check values in correct range
	  }
      catch (java.text.ParseException e)
	  {
	      return false; // parsing failed
	  }
  }
}
