/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.laws;

import noope.core.*;
import noope.input.BlockReader;
import noope.input.BRLoadingException;
import noope.entities.Entity;

/** This is an abstract subclass of Law used to model a Law that acts on a pair of Entitys, 
    i.e. the net force exerted on an entity by this law is the sum of the forces exerted on it by 
    all other entitys. It must be suitable to consider the entitys acted on to be
    points of mass.

    To create a subclass, you have to override actOnPair(Entity, Entity), possibly
    constructNewPropertyRecord and the constructor.
    
    @author Miklos Reiter
    @version 28 April 2001 Removed ListIterator by Miklos Reiter for compatibility with Java 1.1
*/
abstract public class ParticlePairLaw extends Law
{
    /** Just calls the inherited constructor. */
    public ParticlePairLaw(BlockReader br, int lawnumber, Physics phys)
     						throws BRLoadingException
    {
	super(br, lawnumber, phys);
    }

    /** Apply law to all pairs of entities. It just calls actOnPair(Entity, Entity) on all
	pairs of distinct entitys in Entities. If you want to create a subclass to model
	a particular law, you should override actOnPair, not act.
    */
    public void act()
    // for all i != j in e: actOnPair(i, j);
    {
	Entities e = physics.getEntities();
	if (e.size() <= 1) return; // nothing to do
	Entity ent = (Entity)e.get(0), other = (Entity)e.get(0); // appease compiler
      	// get iterator i pointing at first element
	/*
	for (ListIterator i = e.listIterator(); i.hasNext();)
	    {
		ent = (Entity)i.next();
		for (ListIterator j = e.listIterator(i.nextIndex()); j.hasNext(); )
		    {
			other = (Entity)j.next();
			actOnPair(ent, other);
		    }
	    }
	*/
	int i, j;
	for (i = 0; i < e.size(); i++)
	    {
		ent = (Entity)e.elementAt(i);
		for (j = i+1; j < e.size(); j++)
		    {
			other = (Entity)e.elementAt(j);
			actOnPair(ent, other);
		    }
	    }
    }

    /** Applies the law to the two parameters. This should be overridden in subclasses. */
    abstract public void actOnPair(Entity ent1, Entity ent2);
}
