/**
 * Copyright (C) 2001  Miklos Reiter (see the file README for details on
 * how to contact me)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package noope.output;

import java.awt.*;
import java.awt.event.*;

/**
 *
 * This class provides a simple modal MessageBox that can be used to display information to the
 * user.
 *
 * Maybe there is an AWT MessageBox (There ought to be one!), but I've spent half of today looking
 * for it and didn't find it, so I decided I'd implement my own.
 *
 * @author Miklos Reiter
 * @version 26-Apr-2001
 *
 */
public class MessageBox extends Dialog
{
    // Removed because not supported by Java 1.1
    /**
     * Constructs a MessageBox with the given Dialog as its owner and the given title and message.
     */
    /*protected MessageBox(Dialog owner, String title, String message)
	{
	    super(owner, title, true);
	    setupMessageBox(message);
	}
    */

    /**
     * Constructs a MessageBox with the given Frame as its owner and the given title and message.
     */
    protected MessageBox(Frame owner, String title, String message)
	{
	    super(owner, title, true);
	    setupMessageBox(message);
	}

    /**
     * Sets up the MessageBox (once its constructed), so it displays the given message.
     */
    protected void setupMessageBox(String message)
	{
	    setLayout(new BorderLayout());
	    add(new Label(message), BorderLayout.CENTER);
	    Button okButton = new Button("OK");
	    okButton.addActionListener(new java.awt.event.ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			okButton_pressed(e);
		    }
		});
	    add(okButton, BorderLayout.SOUTH);
	    addWindowListener(new WindowAdapter()
		{
		    public void windowClosing(WindowEvent e)
		    {
			okButton_pressed(null);
		    }
		});
	}
    
    /**
     * Called when the OK button on the dialog is pressed. Hides the dialog.
     */
    protected void okButton_pressed(ActionEvent e)
	{
	    // hide dialog
	    setVisible(false);
	}

    // Removed because not supported by Java 1.1
    /** 
     * Displays a MessageBox with the specifided owner, title and message.
     */
    /*    public static void displayMessage(Dialog owner, String title, String message)
	{
	    MessageBox mb = new MessageBox(owner, title, message);
	    mb.pack();
	    mb.setResizable(false);
	    mb.setVisible(true);
	    mb.dispose();
	}
    */

    /** 
     * Displays a MessageBox with the specifided owner, title and message.
     */
    public static void displayMessage(Frame owner, String title, String message)
	{
	    MessageBox mb = new MessageBox(owner, title, message);
	    mb.pack();
	    mb.setResizable(false);
	    mb.setVisible(true);
	    mb.dispose();
	}
}
